# -*- coding: utf-8 -*-

import random

from cached_property import cached_property

import docker
import docker.errors
import docker.utils


class DockerRunner(object):
    def __init__(self, ctx, default_platform=None):
        self.ctx = ctx
        self.default_platform = default_platform

        self.network = None
        self.running_containers = []

    @cached_property
    def client(self):
        return docker.Client(
            version='auto',
            **docker.utils.kwargs_from_env(assert_hostname=False))

    def setup(self):
        self.pull_required_images()

        inventory_lines = []

        for host, container in self.start_containers():
            additional_config = self.additional_config_for_host(host, container)
            inventory_hostname = host.vars()['inventory_hostname']
            inventory_host_options = \
                ' '.join('{0}="{1}"'.format(key, value)
                         for key, value in additional_config.items())
            inventory_line = ' '.join((inventory_hostname, inventory_host_options))
            inventory_lines.append(inventory_line)

        inventory_content = '\n'.join(inventory_lines)

        self.ctx.extended_inventory_path.join('goodplay').write(inventory_content)

    def pull_required_images(self):
        required_images = set()

        for host in self.ctx.inventory.hosts():
            required_image = self.get_docker_image_for_host(host)

            if required_image:
                required_images.add(required_image)

        # TODO: define how to disable this caching and always pull latest
        for required_image in required_images:
            try:
                self.client.inspect_image(required_image)
            except docker.errors.NotFound:
                self.client.pull(required_image)

    def get_docker_image_for_host(self, host):
        host_vars = host.vars()
        goodplay_platform = host_vars.get('goodplay_platform')

        if goodplay_platform == '*':
            return self.default_platform.image
        elif goodplay_platform:
            platform = self.ctx.platform_manager.find_by_id(goodplay_platform)

            if not platform:
                raise Exception(
                    "goodplay_platform '{0}' specified in inventory for "
                    "host '{1}' not found in .goodplay.yml".format(
                        goodplay_platform, host_vars['inventory_hostname']))

            return platform.image

        return host_vars.get('goodplay_image')

    def start_containers(self):
        for host in self.ctx.inventory.hosts():
            if not self.get_docker_image_for_host(host):
                continue

            if not self.network:
                network_name = u'goodplay{0}'.format(random.getrandbits(24))[:14]
                self.network = self.client.create_network(network_name, driver='bridge')

            container = self.start_container(host)
            self.running_containers.append(container)

            yield host, container

    def start_container(self, host):
        hostname = host.vars()['inventory_hostname']
        image = self.get_docker_image_for_host(host)

        is_fqhn = '.' in hostname
        domainname = hostname.split('.', 1)[-1] if is_fqhn else None
        dns_search = [domainname] if domainname else None

        # cap_add probably needed when supporting KVM

        host_config = self.client.create_host_config(
            dns_search=dns_search
        )

        container = self.client.create_container(
            image=image,
            hostname=hostname,
            domainname=domainname,
            detach=True,
            tty=True,
            host_config=host_config
        )

        self.client.connect_container_to_network(container, self.network['Id'], aliases=[hostname])

        self.client.start(container)

        return container

    def additional_config_for_host(self, host, container):
        return dict(
            ansible_connection='docker',
            ansible_host=container['Id'],
        )

    def teardown(self):
        # kill and remove containers
        for container in self.running_containers:
            self.client.remove_container(container, force=True)

        if self.network:
            self.client.remove_network(self.network['Id'])



# -*- coding: utf-8 -*-

import yaml

from .inventory import Inventory  # noqa: F401
from .playbook import Playbook  # noqa: F401


def is_test_playbook_file(path):
    has_test_prefix = path.basename.startswith('test_')

    return has_test_prefix and is_playbook_file(path)


def is_playbook_file(path):
    is_yaml_file = path.ext == '.yml'

    if is_yaml_file:
        yaml_file_content = yaml.safe_load(path.read())
        yaml_file_is_ansible_playbook = is_playbook_content(yaml_file_content)

        return yaml_file_is_ansible_playbook

    return False


def is_playbook_content(content):
    return isinstance(content, list) \
        and len(content) \
        and isinstance(content[0], dict) \
        and (content[0].get('hosts') or content[0].get('include'))



# -*- coding: utf-8 -*-

from goodplay import config


def test_find_config_path_missing(tmpdir):
    playbook_path = tmpdir.join('test_playbook.yml').ensure(file=True)

    assert config.find_config_path(playbook_path) is None


def test_find_config_path_does_not_match_file_with_long_yaml_extension(tmpdir):
    playbook_path = tmpdir.join('test_playbook.yml').ensure(file=True)
    tmpdir.join('.goodplay.yaml').ensure(file=True)

    assert config.find_config_path(playbook_path) is None


def test_find_config_path_does_not_match_directory(tmpdir):
    playbook_path = tmpdir.join('test_playbook.yml').ensure(file=True)
    tmpdir.join('.goodplay.yml').ensure(dir=True)

    assert config.find_config_path(playbook_path) is None


def test_find_config_path_beside_playbook_path(tmpdir):
    playbook_path = tmpdir.join('test_playbook.yml').ensure(file=True)
    expected_config_path = tmpdir.join('.goodplay.yml').ensure(file=True)

    assert config.find_config_path(playbook_path) == expected_config_path


def test_find_config_path_in_playbook_path_direct_ancestor_dir(tmpdir):
    playbook_path = tmpdir.join('dir1', 'test_playbook.yml').ensure(file=True)
    expected_config_path = tmpdir.join('.goodplay.yml').ensure(file=True)

    assert config.find_config_path(playbook_path) == expected_config_path


def test_find_config_path_in_playbook_path_some_ancestor_dir(tmpdir):
    playbook_path = tmpdir.join(
        'dir1', 'dir2', 'dir3', 'test_playbook.yml').ensure(file=True)
    expected_config_path = tmpdir.join('.goodplay.yml').ensure(file=True)

    assert config.find_config_path(playbook_path) == expected_config_path


def test_find_config_path_uses_depth_first(tmpdir):
    playbook_path = tmpdir.join(
        'dir1', 'dir2', 'dir3', 'test_playbook.yml').ensure(file=True)

    # intentionally created config file which should not be matched
    tmpdir.join('dir1', '.goodplay.yml').ensure(file=True)

    expected_config_path = tmpdir.join(
        'dir1', 'dir2', '.goodplay.yml').ensure(file=True)

    assert config.find_config_path(playbook_path) == expected_config_path


def test_get_goodplay_config_defaults(tmpdir):
    playbook_path = tmpdir.join('test_playbook.yml').ensure(file=True)

    cfg = config.get_goodplay_config(playbook_path)

    assert isinstance(cfg, dict)
    assert cfg.get('platforms') == []


def test_get_goodplay_config(tmpdir):
    playbook_path = tmpdir.join('test_playbook.yml').ensure(file=True)

    tmpdir.join('.goodplay.yml').write('''---
platforms:
  - name: EL
    version: 7
    image: centos:centos7
''', ensure=True)

    cfg = config.get_goodplay_config(playbook_path)

    assert isinstance(cfg, dict)
    assert isinstance(cfg['platforms'], list)
    assert cfg['platforms'] == [
        dict(name='EL', version=7, image='centos:centos7')]



# -*- coding: utf-8 -*-

import docker
import docker.errors
import docker.utils
import pytest

from goodplay_helpers import smart_create

pytestmark = pytest.mark.integration


@pytest.fixture
def docker_client(mocker):
    proxy_method_names = (
        'create_container',
        'inspect_image',
        'pull',
        'remove_container',
        'start',
    )

    client = docker.Client(
        version='auto',
        **docker.utils.kwargs_from_env(assert_hostname=False))

    for proxy_method_name in proxy_method_names:
        origin_method = getattr(client, proxy_method_name)
        mocker.patch.object(client, proxy_method_name).side_effect = origin_method

    mocker.patch('docker.Client').return_value = client

    return client


@pytest.fixture
def docker_client_unavailable(mocker):
    client = docker.Client(base_url='unix://tmp/docker.unavailable.sock')

    mocker.patch('docker.Client').return_value = client

    return client


def test_docker_is_not_initialized_when_not_used(testdir, mocker):
    mock_docker_client = mocker.patch('docker.Client', autospec=True)

    smart_create(testdir.tmpdir, '''
    ## inventory
    default ansible_connection=local

    ## test_playbook.yml
    - hosts: default
      gather_facts: no
      tasks:
        - name: host is reachable
          ping:
          tags: test
    ''')

    result = testdir.inline_run('-s')
    result.assertoutcome(passed=1)

    assert not mock_docker_client.called


def test_docker_is_not_initialized_when_collect_only(testdir, mocker):
    mock_docker_client = mocker.patch('docker.Client', autospec=True)

    smart_create(testdir.tmpdir, '''
    ## inventory
    guesthostname goodplay_image=busybox:latest

    ## test_playbook.yml
    - hosts: guesthostname
      gather_facts: no
      tasks:
        - name: task 1
          raw: "ls -la /"
          changed_when: False
          tags: test
    ''')

    items, result = testdir.inline_genitems()
    result.assertoutcome()
    assert len(items) == 1
    assert items[0].name == 'task 1'

    assert not mock_docker_client.called


def test_docker_used_and_not_available(testdir, docker_client_unavailable):
    smart_create(testdir.tmpdir, '''
    ## inventory
    guesthostname goodplay_image=busybox:latest

    ## test_playbook.yml
    - hosts: guesthostname
      gather_facts: no
      tasks:
        - name: task 1
          raw: "ls -la /"
          changed_when: False
          tags: test
    ''')

    result = testdir.inline_run('-s')
    result.assertoutcome(failed=1)


def test_docker_pull_called_when_image_does_not_exist(testdir, docker_client):
    docker_client.inspect_image.side_effect = \
        docker.errors.NotFound('image not found', None,
                               explanation='simulate missing docker image')

    smart_create(testdir.tmpdir, '''
    ## inventory
    guesthostname goodplay_image=busybox:latest ansible_user=root

    ## test_playbook.yml
    - hosts: guesthostname
      gather_facts: no
      tasks:
        - name: task 1
          raw: "ls -la /"
          changed_when: False
          tags: test
    ''')

    result = testdir.inline_run('-s')
    result.assertoutcome(passed=1)

    docker_client.inspect_image.assert_called_once_with('busybox:latest')
    docker_client.pull.assert_called_once_with('busybox:latest')


def test_docker_pull_is_called_once_per_image_when_multiple_times(testdir, docker_client):
    docker_client.inspect_image.side_effect = \
        docker.errors.NotFound('image not found', None,
                               explanation='simulate missing docker image')

    smart_create(testdir.tmpdir, '''
    ## inventory
    guesthostname1 goodplay_image=busybox:latest ansible_user=root
    guesthostname2 goodplay_image=busybox:latest ansible_user=root

    ## test_playbook.yml
    - hosts: guesthostname1:guesthostname2
      gather_facts: no
      tasks:
        - name: task 1
          raw: "ls -la /"
          changed_when: False
          tags: test
    ''')

    result = testdir.inline_run('-s')
    result.assertoutcome(passed=1)

    docker_client.inspect_image.assert_called_once_with('busybox:latest')
    docker_client.pull.assert_called_once_with('busybox:latest')


def test_docker_pull_not_called_when_image_already_exists(testdir, docker_client):
    # ensure image is present
    docker_client.pull('busybox:latest')
    docker_client.pull.reset_mock()

    smart_create(testdir.tmpdir, '''
    ## inventory
    guesthostname goodplay_image=busybox:latest ansible_user=root

    ## test_playbook.yml
    - hosts: guesthostname
      gather_facts: no
      tasks:
        - name: task 1
          raw: "ls -la /"
          changed_when: False
          tags: test
    ''')

    result = testdir.inline_run('-s')
    result.assertoutcome(passed=1)

    docker_client.inspect_image.assert_called_once_with('busybox:latest')
    docker_client.pull.assert_not_called()


def test_failed_when_goodplay_platform_cannot_be_resolved(testdir, docker_client):
    smart_create(testdir.tmpdir, '''
    ## inventory
    guesthostname goodplay_platform=thename:theversion

    ## test_playbook.yml
    - hosts: guesthostname
      gather_facts: no
      tasks:
        - name: task 1
          raw: "ls -la /"
          changed_when: False
          tags: test
    ''')

    result = testdir.inline_run('-s')
    result.assertoutcome(failed=1)

    message = "goodplay_platform 'thename:theversion' specified in inventory for host " \
        "'guesthostname' not found in .goodplay.yml"
    assert message in str(result.getfailures()[0].longrepr)


def test_docker_pull_called_with_resolved_goodplay_platform(testdir, docker_client):
    docker_client.inspect_image.side_effect = \
        docker.errors.NotFound('image not found', None,
                               explanation='simulate missing docker image')

    smart_create(testdir.tmpdir, '''
    ## .goodplay.yml
    platforms:
      - name: thename
        version: theversion
        image: busybox:latest

    ## inventory
    guesthostname goodplay_platform=thename:theversion ansible_user=root

    ## test_playbook.yml
    - hosts: guesthostname
      gather_facts: no
      tasks:
        - name: task 1
          raw: "ls -la /"
          changed_when: False
          tags: test
    ''')

    result = testdir.inline_run('-s')
    result.assertoutcome(passed=1)

    docker_client.inspect_image.assert_called_once_with('busybox:latest')
    docker_client.pull.assert_called_once_with('busybox:latest')


def test_started_docker_container_is_removed_after_successful_run(testdir, docker_client):
    smart_create(testdir.tmpdir, '''
    ## inventory
    guesthostname goodplay_image=busybox:latest ansible_user=root

    ## test_playbook.yml
    - hosts: guesthostname
      gather_facts: no
      tasks:
        - name: task 1
          raw: "ls -la /"
          changed_when: False
          tags: test
    ''')

    result = testdir.inline_run('-s')
    result.assertoutcome(passed=1)

    assert docker_client.start.call_count == 1

    started_container_id = docker_client.start.call_args[0][0]
    docker_client.remove_container.assert_called_once_with(
        started_container_id, force=True)


def test_started_docker_container_is_removed_after_failed_run(testdir, docker_client):
    smart_create(testdir.tmpdir, '''
    ## inventory
    guesthostname goodplay_image=busybox:latest ansible_user=root

    ## test_playbook.yml
    - hosts: guesthostname
      gather_facts: no
      tasks:
        - name: intentionally failing task
          raw: who
          failed_when: True
          tags: test
    ''')

    result = testdir.inline_run('-s')
    result.assertoutcome(failed=1)

    assert docker_client.start.call_count == 1

    started_container_id = docker_client.start.call_args[0][0]
    docker_client.remove_container.assert_called_once_with(
        started_container_id, force=True)


def test_group_vars_directory_beside_inventory_file_is_incorporated(testdir):
    smart_create(testdir.tmpdir, '''
    ## inventory
    guesthostname goodplay_image=busybox:latest ansible_user=root

    ## group_vars/all
    hello: world

    ## test_playbook.yml
    - hosts: guesthostname
      gather_facts: no
      tasks:
        - name: task 1
          raw: 'echo -n {{ hello }}'
          register: echo_result
          failed_when: echo_result.stdout != 'world'
          changed_when: False
          tags: test
    ''')

    result = testdir.inline_run('-s')
    result.assertoutcome(passed=1)


def test_group_vars_directory_beside_inventory_directory_is_incorporated(testdir):
    smart_create(testdir.tmpdir, '''
    ## inventory/static
    guesthostname goodplay_image=busybox:latest ansible_user=root

    ## inventory/group_vars/all
    hello: world

    ## test_playbook.yml
    - hosts: guesthostname
      gather_facts: no
      tasks:
        - name: task 1
          raw: 'echo -n {{ hello }}'
          register: echo_result
          failed_when: echo_result.stdout != 'world'
          changed_when: False
          tags: test
    ''')

    result = testdir.inline_run('-s')
    result.assertoutcome(passed=1)


def test_host_vars_directory_beside_inventory_file_is_incorporated(testdir):
    smart_create(testdir.tmpdir, '''
    ## inventory
    guesthostname goodplay_image=busybox:latest ansible_user=root

    ## host_vars/guesthostname
    hello: world

    ## test_playbook.yml
    - hosts: guesthostname
      gather_facts: no
      tasks:
        - name: task 1
          raw: 'echo -n {{ hello }}'
          register: echo_result
          failed_when: echo_result.stdout != 'world'
          changed_when: False
          tags: test
    ''')

    result = testdir.inline_run('-s')
    result.assertoutcome(passed=1)


def test_host_vars_directory_beside_inventory_directory_is_incorporated(testdir):
    smart_create(testdir.tmpdir, '''
    ## inventory/static
    guesthostname goodplay_image=busybox:latest ansible_user=root

    ## inventory/host_vars/guesthostname
    hello: world

    ## test_playbook.yml
    - hosts: guesthostname
      gather_facts: no
      tasks:
        - name: task 1
          raw: 'echo -n {{ hello }}'
          register: echo_result
          failed_when: echo_result.stdout != 'world'
          changed_when: False
          tags: test
    ''')

    result = testdir.inline_run('-s')
    result.assertoutcome(passed=1)


def test_role_with_goodplay_platform_wildcard(testdir, docker_client):
    smart_create(testdir.tmpdir, '''
    ## local-role-base/role1/meta/main.yml
    galaxy_info:
      author: John Doe
      platforms:
        - name: EL
          versions:
            - 6
            - 7
    dependencies: []

    ## local-role-base/role1/tasks/main.yml
    - ping:

    ## local-role-base/role1/tests/.goodplay.yml
    platforms:
      - name: EL
        version: 6
        image: centos:centos6

      - name: EL
        version: 7
        image: centos:centos7

    ## local-role-base/role1/tests/inventory
    default goodplay_platform=* ansible_user=root

    ## local-role-base/role1/tests/test_playbook.yml
    - hosts: default
      gather_facts: no
      tasks:
        - name: host is reachable
          ping:
          tags: test
    ''')

    result = testdir.inline_run('-s')
    result.assertoutcome(passed=2)

    assert docker_client.create_container.call_count == 2

    _, _, kwargs = docker_client.create_container.mock_calls[0]
    assert kwargs['image'] == 'centos:centos6'

    _, _, kwargs = docker_client.create_container.mock_calls[1]
    assert kwargs['image'] == 'centos:centos7'

    assert docker_client.start.call_count == 2


def test_hosts_without_domain_can_resolve_each_other(testdir):
    smart_create(testdir.tmpdir, '''
    ## inventory
    guesthost1 goodplay_image=busybox:latest ansible_user=root
    guesthost2 goodplay_image=busybox:latest ansible_user=root

    ## test_playbook.yml
    - hosts: guesthost1
      gather_facts: no
      tasks:
        - name: assert guesthost2 is reachable
          raw: ping -c 1 guesthost2
          changed_when: False
          tags: test

    - hosts: guesthost2
      gather_facts: no
      tasks:
        - name: assert guesthost1 is reachable
          raw: ping -c 1 guesthost1
          changed_when: False
          tags: test
    ''')

    result = testdir.inline_run('-s')
    result.assertoutcome(passed=2)


def test_hosts_with_domain_can_resolve_each_other(testdir):
    smart_create(testdir.tmpdir, '''
    ## inventory
    host1.domain1.tld goodplay_image=busybox:latest ansible_user=root
    host2.domain1.tld goodplay_image=busybox:latest ansible_user=root
    host1.domain2.tld goodplay_image=busybox:latest ansible_user=root

    ## test_playbook.yml
    - hosts: host1.domain1.tld
      gather_facts: no
      tasks:
        # host1.domain1.tld assertions
        - name: assert host1.domain1.tld can be resolved on host1.domain1.tld
          raw: ping -c 1 host1.domain1.tld
          changed_when: False
          tags: test

        - name: assert host1 can be resolved on host1.domain1.tld
          raw: ping -c 1 host1
          changed_when: False
          tags: test

        # host2.domain1.tld assertions
        - name: assert host2.domain1.tld can be resolved on host1.domain1.tld
          raw: ping -c 1 host2.domain1.tld
          changed_when: False
          tags: test

        - name: assert host2 can be resolved on host1.domain1.tld
          raw: ping -c 1 host2
          changed_when: False
          tags: test

        # host1.domain2.tld assertions
        - name: assert host1.domain2.tld can be resolved on host1.domain1.tld
          raw: ping -c 1 host1.domain2.tld
          changed_when: False
          tags: test

    - hosts: host2.domain1.tld
      gather_facts: no
      tasks:
        # host1.domain1.tld assertions
        - name: assert host1.domain1.tld can be resolved on host2.domain1.tld
          raw: ping -c 1 host1.domain1.tld
          changed_when: False
          tags: test

        - name: assert host1 can be resolved on host2.domain1.tld
          raw: ping -c 1 host1
          changed_when: False
          tags: test

        # host2.domain1.tld assertions
        - name: assert host2.domain1.tld can be resolved on host2.domain1.tld
          raw: ping -c 1 host2.domain1.tld
          changed_when: False
          tags: test

        - name: assert host2 can be resolved on host2.domain1.tld
          raw: ping -c 1 host2
          changed_when: False
          tags: test

        # host1.domain2.tld assertions
        - name: assert host1.domain2.tld can be resolved on host2.domain1.tld
          raw: ping -c 1 host1.domain2.tld
          changed_when: False
          tags: test

    - hosts: host1.domain2.tld
      gather_facts: no
      tasks:
        # host1.domain1.tld assertions
        - name: assert host1.domain1.tld can be resolved on host1.domain2.tld
          raw: ping -c 1 host1.domain1.tld
          changed_when: False
          tags: test

        # host2.domain1.tld assertions
        - name: assert host2.domain1.tld can be resolved on host1.domain2.tld
          raw: ping -c 1 host2.domain1.tld
          changed_when: False
          tags: test

        - name: assert host2 cannot be resolved on host1.domain2.tld
          raw: ping -c 1 host2
          register: ping_result
          changed_when: False
          failed_when: "ping_result.rc != 1"
          tags: test

        # host1.domain2.tld assertions
        - name: assert host1.domain2.tld can be resolved on host1.domain2.tld
          raw: ping -c 1 host1.domain2.tld
          changed_when: False
          tags: test

        - name: assert host1 can be resolved on host1.domain2.tld
          raw: ping -c 1 host1
          changed_when: False
          tags: test
    ''')

    result = testdir.inline_run('-s')
    result.assertoutcome(passed=15)


def test_hosts_can_resolve_google_com_domain(testdir):
    smart_create(testdir.tmpdir, '''
    ## inventory
    guesthost goodplay_image=busybox:latest ansible_user=root

    ## test_playbook.yml
    - hosts: guesthost
      gather_facts: no
      tasks:
        - name: assert google.com domain is reachable
          raw: ping -c 1 google.com
          changed_when: False
          tags: test
    ''')

    result = testdir.inline_run('-s')
    result.assertoutcome(passed=1)



# -*- coding: utf-8 -*-

from goodplay.ansible_support import Inventory, Playbook
from goodplay.context import GoodplayContext, Platform, PlatformManager


# inventory_path

def test_inventory_path_file_found_in_playbook_path_dir(tmpdir):
    playbook_path = tmpdir.join('test_playbook.yml')
    playbook_path.ensure()

    inventory_path = tmpdir.join('inventory')
    inventory_path.ensure()

    ctx = GoodplayContext(playbook_path)

    assert ctx.inventory_path == inventory_path


def test_inventory_path_dir_found_in_playbook_path_dir(tmpdir):
    playbook_path = tmpdir.join('test_playbook.yml')
    playbook_path.ensure()

    inventory_path = tmpdir.join('inventory')
    inventory_path.ensure(dir=True)

    ctx = GoodplayContext(playbook_path)

    assert ctx.inventory_path == inventory_path


def test_inventory_path_is_none_when_not_in_playbook_path_dir(tmpdir):
    playbook_path = tmpdir.join('test_playbook.yml')
    playbook_path.ensure()

    ctx = GoodplayContext(playbook_path)

    assert ctx.inventory_path is None


def test_inventory_path_is_none_when_inventory_path_in_parent_dir(tmpdir):
    playbook_path = tmpdir.join('parent_dir', 'test_playbook.yml')
    playbook_path.ensure()

    inventory_path = tmpdir.join('inventory')
    inventory_path.ensure()

    ctx = GoodplayContext(playbook_path)

    assert ctx.inventory_path is None


def test_inventory_path_is_none_when_inventory_path_in_sub_dir(tmpdir):
    playbook_path = tmpdir.join('test_playbook.yml')
    playbook_path.ensure()

    inventory_path = tmpdir.join('sub_dir', 'inventory')
    inventory_path.ensure()

    ctx = GoodplayContext(playbook_path)

    assert ctx.inventory_path is None


def test_inventory_path_is_cached(tmpdir):
    playbook_path = tmpdir.join('test_playbook.yml')
    playbook_path.ensure()

    inventory_path = tmpdir.join('inventory')
    inventory_path.ensure()

    ctx = GoodplayContext(playbook_path)

    first_result = ctx.inventory_path
    assert first_result is not None
    assert id(ctx.inventory_path) == id(first_result)


# inventory

def test_inventory_is_none_when_inventory_path_is_none(tmpdir):
    playbook_path = tmpdir.join('test_playbook.yml')
    playbook_path.ensure()

    ctx = GoodplayContext(playbook_path)
    ctx.inventory_path = None

    assert ctx.inventory is None


def test_inventory_class(tmpdir):
    inventory_path = tmpdir.join('inventory')
    inventory_path.write('host1', ensure=True)

    playbook_path = tmpdir.join('test_playbook.yml')
    playbook_path.ensure()

    ctx = GoodplayContext(playbook_path)

    assert ctx.inventory.__class__ == Inventory


def test_inventory_is_cached(tmpdir):
    inventory_path = tmpdir.join('inventory')
    inventory_path.write('host1', ensure=True)

    playbook_path = tmpdir.join('test_playbook.yml')
    playbook_path.ensure()

    ctx = GoodplayContext(playbook_path)

    first_result = ctx.inventory
    assert first_result is not None
    assert id(ctx.inventory) == id(first_result)


# playbook

def test_playbook_is_none_when_inventory_path_is_none(tmpdir):
    playbook_path = tmpdir.join('test_playbook.yml')
    playbook_path.ensure()

    ctx = GoodplayContext(playbook_path)
    ctx.inventory_path = None

    assert ctx.playbook is None


def test_playbook_class(tmpdir):
    inventory_path = tmpdir.join('inventory')
    inventory_path.write('host1', ensure=True)

    playbook_path = tmpdir.join('test_playbook.yml')
    playbook_path.write('''---
- hosts: host1
  tasks:
    - name: task 1
      ping:
      tags: test
''', ensure=True)

    ctx = GoodplayContext(playbook_path)

    assert ctx.playbook.__class__ == Playbook


def test_playbook_is_cached(tmpdir):
    inventory_path = tmpdir.join('inventory')
    inventory_path.write('host1', ensure=True)

    playbook_path = tmpdir.join('test_playbook.yml')
    playbook_path.write('''---
- hosts: host1
  tasks:
    - name: task 1
      ping:
      tags: test
''', ensure=True)

    ctx = GoodplayContext(playbook_path)

    first_result = ctx.playbook
    assert first_result is not None
    assert id(ctx.playbook) == id(first_result)


# role_path

def test_role_path_is_none_when_plain_test_playbook(tmpdir):
    playbook_path = tmpdir.join('test_playbook.yml')
    playbook_path.ensure()

    ctx = GoodplayContext(playbook_path)

    assert ctx.role_path is None


def test_role_path_is_none_when_tests_parent_dir_only(tmpdir):
    playbook_path = tmpdir.join('tests', 'test_playbook.yml')
    playbook_path.ensure()

    ctx = GoodplayContext(playbook_path)

    assert ctx.role_path is None


def test_role_path_is_none_when_meta_dir_beside_tests_parent_dir_only(tmpdir):
    tmpdir.join('meta').ensure(dir=True)

    playbook_path = tmpdir.join('tests', 'test_playbook.yml')
    playbook_path.ensure()

    ctx = GoodplayContext(playbook_path)

    assert ctx.role_path is None


def test_role_path_is_none_when_meta_dir_with_main_yml_dir_beside_tests_parent_dir(tmpdir):
    tmpdir.join('meta', 'main.yml').ensure(dir=True)

    playbook_path = tmpdir.join('tests', 'test_playbook.yml')
    playbook_path.ensure()

    ctx = GoodplayContext(playbook_path)

    assert ctx.role_path is None


def test_role_path_is_none_when_meta_dir_with_main_yml_file_beside_playbook_path_dir(tmpdir):
    tmpdir.join('meta', 'main.yml').ensure()

    playbook_path = tmpdir.join('test_playbook.yml')
    playbook_path.ensure()

    ctx = GoodplayContext(playbook_path)

    assert ctx.role_path is None


def test_role_path_when_meta_dir_with_main_yml_file_beside_tests_parent_dir(tmpdir):
    tmpdir.join('meta', 'main.yml').ensure()

    playbook_path = tmpdir.join('tests', 'test_playbook.yml')
    playbook_path.ensure()

    ctx = GoodplayContext(playbook_path)

    assert ctx.role_path == tmpdir


def test_role_path_when_meta_dir_with_main_yml_file_beside_tests_ancestor_dir(tmpdir):
    tmpdir.join('meta', 'main.yml').ensure()

    playbook_path = tmpdir.join('tests', 'sub_dir1', 'sub_dir2', 'test_playbook.yml')
    playbook_path.ensure()

    ctx = GoodplayContext(playbook_path)

    assert ctx.role_path == tmpdir


def test_role_path_is_cached(tmpdir):
    tmpdir.join('meta', 'main.yml').ensure()

    playbook_path = tmpdir.join('tests', 'test_playbook.yml')
    playbook_path.ensure()

    ctx = GoodplayContext(playbook_path)

    first_result = ctx.role_path
    assert first_result is not None
    assert id(ctx.role_path) == id(first_result)


# is_role_playbook

def test_is_role_playbook_is_false_when_role_path_is_none(tmpdir):
    playbook_path = tmpdir.join('test_playbook.yml')
    playbook_path.ensure()

    ctx = GoodplayContext(playbook_path)
    ctx.role_path = None

    assert ctx.is_role_playbook is False


def test_is_role_playbook_is_true_when_role_path_is_not_none(tmpdir):
    playbook_path = tmpdir.join('test_playbook.yml')
    playbook_path.ensure()

    ctx = GoodplayContext(playbook_path)
    ctx.role_path = tmpdir

    assert ctx.is_role_playbook is True


# role_meta_info

def test_role_meta_info_reads_meta_dir_main_yml_file(tmpdir):
    tmpdir.join('meta', 'main.yml').write('''---
hello: world
''', ensure=True)

    playbook_path = tmpdir.join('tests', 'test_playbook.yml')
    playbook_path.ensure()

    ctx = GoodplayContext(playbook_path)

    assert ctx.role_meta_info == dict(hello='world')


def test_role_meta_info_is_cached(tmpdir):
    tmpdir.join('meta', 'main.yml').write('''---
galaxy_info:
''', ensure=True)

    playbook_path = tmpdir.join('tests', 'test_playbook.yml')
    playbook_path.ensure()

    ctx = GoodplayContext(playbook_path)

    first_result = ctx.role_meta_info
    assert first_result is not None
    assert id(ctx.role_meta_info) == id(first_result)


# platform

def test_platform_base_attributes():
    platform = Platform(name='ubuntu', version='trusty')

    assert platform.name == 'ubuntu'
    assert platform.version == 'trusty'


def test_platform_equal():
    platform1 = Platform(name='ubuntu', version='trusty')
    platform2 = Platform(name='ubuntu', version='trusty')

    assert platform1 == platform2
    assert not platform1 != platform2


def test_platform_version_int_and_string_are_comparable():
    platform1 = Platform(name='EL', version=6)
    platform2 = Platform(name='EL', version='6')

    assert platform1 == platform2
    assert not platform1 != platform2


def test_platform_equal_with_different_kwargs():
    platform1 = Platform(name='ubuntu', version='trusty')
    platform2 = Platform(name='ubuntu', version='trusty', some_param=1234)

    assert platform1 == platform2
    assert not platform1 != platform2


# platform manager

def test_platform_manager_find_by_id_found():
    available_platform = Platform('ubuntu', 'trusty')
    available_platforms = [available_platform]
    platform_manager = PlatformManager(available_platforms)

    assert id(platform_manager.find_by_id('ubuntu:trusty')) == id(available_platform)


def test_platform_manager_find_by_id_not_found():
    available_platform = Platform('ubuntu', 'trusty')
    available_platforms = [available_platform]
    platform_manager = PlatformManager(available_platforms)

    assert platform_manager.find_by_id('ubuntu:precise') is None


def test_platform_manager_find_by_id_without_colon():
    available_platform = Platform('ubuntu', 'trusty')
    available_platforms = [available_platform]
    platform_manager = PlatformManager(available_platforms)

    assert platform_manager.find_by_id('ubuntutrusty') is None


def test_platform_manager_find_by_name_and_version_found():
    available_platform = Platform('ubuntu', 'trusty')
    available_platforms = [available_platform]
    platform_manager = PlatformManager(available_platforms)

    found_platform = platform_manager.find_by_name_and_version('ubuntu', 'trusty')
    assert id(found_platform) == id(available_platform)


def test_platform_manager_find_by_name_and_version_not_found():
    available_platform = Platform('ubuntu', 'trusty')
    available_platforms = [available_platform]
    platform_manager = PlatformManager(available_platforms)

    assert platform_manager.find_by_name_and_version('ubuntu', 'precise') is None


def test_platform_manager_selected_platforms_initial_empty():
    available_platforms = [Platform('ubuntu', 'trusty')]
    platform_manager = PlatformManager(available_platforms)

    assert len(platform_manager.selected_platforms) == 0


def test_platform_manager_select_platform_by_name_and_version_adds_platform_to_selected():
    available_platform = Platform('ubuntu', 'trusty')
    available_platforms = [available_platform]
    platform_manager = PlatformManager(available_platforms)

    assert len(platform_manager.selected_platforms) == 0

    platform_manager.select_platform_by_name_and_version('ubuntu', 'trusty')

    assert len(platform_manager.selected_platforms) == 1
    assert id(platform_manager.selected_platforms[0]) == id(available_platform)


def test_platform_manager_select_platform_by_name_and_version_ignores_already_selected():
    available_platform = Platform('ubuntu', 'trusty')
    available_platforms = [available_platform]
    platform_manager = PlatformManager(available_platforms)

    assert len(platform_manager.selected_platforms) == 0

    platform_manager.select_platform_by_name_and_version('ubuntu', 'trusty')
    platform_manager.select_platform_by_name_and_version('ubuntu', 'trusty')

    assert len(platform_manager.selected_platforms) == 1
    assert id(platform_manager.selected_platforms[0]) == id(available_platform)


def test_platform_manager_select_platform_by_name_and_version_ignores_non_available():
    available_platforms = [Platform('ubuntu', 'trusty')]
    platform_manager = PlatformManager(available_platforms)

    assert len(platform_manager.selected_platforms) == 0

    platform_manager.select_platform_by_name_and_version('ubuntu', 'precise')

    assert len(platform_manager.selected_platforms) == 0



# -*- coding: utf-8 -*-

import ansible
import ansible.inventory
import ansible.parsing.dataloader
import ansible.utils.vars
import ansible.vars


class EmptyOptions(object):
    extra_vars = []


class Inventory(object):
    def __init__(self, inventory_path):
        self.inventory_path = inventory_path
        self.inventory = self.build_inventory()

    def build_inventory(self):
        self.clear_host_caches()

        loader = ansible.parsing.dataloader.DataLoader()
        variable_manager = ansible.vars.VariableManager()
        variable_manager.extra_vars = ansible.utils.vars.load_extra_vars(
            loader=loader, options=EmptyOptions())

        inventory = ansible.inventory.Inventory(
            loader=loader,
            variable_manager=variable_manager,
            host_list=str(self.inventory_path))
        variable_manager.set_inventory(inventory)

        return inventory

    def clear_host_caches(self):
        # unfortunately we have to reset caches as these are kept as module state
        ansible.inventory.HOSTS_PATTERNS_CACHE.clear()
        ansible.vars.HOSTVARS_CACHE.clear()

    def hosts(self):
        self.clear_host_caches()

        return [Host(host) for host in self.inventory.get_hosts()]


class Host(object):
    def __init__(self, host):
        self.host = host

    def vars(self):
        return self.host.get_vars()



# -*- coding: utf-8 -*-

import json

from ansible.plugins.callback import CallbackBase


class CallbackModule(CallbackBase):
    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = 'notification'
    CALLBACK_NAME = 'goodplay'

    def __init__(self, *args, **kwargs):
        super(CallbackModule, self).__init__(*args, **kwargs)

        self.previous_task = None
        self.previously_ended_task = None
        self.task = None

        self.reset_per_host_outcomes()

    # ansible playbook-specific callback methods

    def v2_playbook_on_task_start(self, task, is_conditional):
        # ensure task is available as attribute on instance
        self.task = task

        self.check_and_handle_playbook_on_task_end()

        if self.is_test_task(self.task):
            self.playbook_on_test_task_start(self.task)

    def check_and_handle_playbook_on_task_end(self):
        if self.previous_task and self.previously_ended_task != self.previous_task:
            self.playbook_on_task_end(self.previous_task)
            self.previously_ended_task = self.previous_task

        self.previous_task = self.task

    def is_test_task(self, task):
        # task can be None for meta tasks (e.g. setup)
        return task and getattr(task, 'name') and 'test' in task.tags

    def playbook_on_test_task_start(self, task):
        self.send_event('test-task-start', name=task.name)

    def send_event(self, event_name, **kwargs):
        event_line_prefix = 'GOODPLAY => '
        event_json = json.dumps(dict(event_name=event_name, data=kwargs))
        msg = event_line_prefix + event_json
        self._display.display(msg)

    def playbook_on_task_end(self, task):
        if self.is_test_task(task):
            self.playbook_on_test_task_end(task)

    def playbook_on_test_task_end(self, task):
        outcome = self.final_test_task_outcome()

        self.send_event('test-task-end', name=task.name, outcome=outcome)
        self.reset_per_host_outcomes()

    def v2_playbook_on_play_start(self, play):
        self.check_and_handle_playbook_on_task_end()

    def v2_playbook_on_stats(self, stats):
        self.check_and_handle_playbook_on_task_end()

    # ansible runner-specific callback methods

    def runner_on_failed(self, host, res, ignore_errors=False):
        if ignore_errors:
            return

        self.handle_failed_result(host, res)

    def runner_on_ok(self, host, res):
        changed = res.get('changed')

        if changed:
            self.handle_changed_result(host, res)
        else:
            self.handle_non_changed_result(host, res)

    def runner_on_skipped(self, host, item=None):
        self.handle_skipped_result(host)

    def runner_on_unreachable(self, host, res):
        self.handle_failed_result(host, res)

    # handle results and outcome

    def reset_per_host_outcomes(self):
        self.per_host_outcomes = {}

    def add_per_host_outcome(self, host, outcome, res=None):
        self.per_host_outcomes[host] = dict(outcome=outcome, res=res)

    def final_test_task_outcome(self):
        if self.per_host_outcomes:
            outcomes = set(x['outcome'] for x in self.per_host_outcomes.values())
            outcome_priority = ('failed', 'skipped', 'passed')
            for outcome in outcome_priority:
                if outcome in outcomes:
                    return outcome

        return 'skipped'

    def handle_changed_result(self, host, res):
        self.check_and_handle_test_task(host, 'failed', res)

    def handle_non_changed_result(self, host, res):
        self.check_and_handle_test_task(host, 'passed', res)

    def handle_skipped_result(self, host):
        self.check_and_handle_test_task(host, 'skipped')

    def handle_failed_result(self, host, res):
        if not self.check_and_handle_test_task(host, 'failed', res):
            self.send_event('error', message=str(res))

    def check_and_handle_test_task(self, host, outcome, res=None):
        if self.is_test_task(self.task):
            self.add_per_host_outcome(host, outcome, res)
            return True
        return False



# -*- coding: utf-8 -*-

import yaml


def get_goodplay_config(playbook_path):
    config_path = find_config_path(playbook_path)

    if config_path:
        return yaml.safe_load(config_path.read())

    return get_default_goodplay_config()


def find_config_path(playbook_path):
    for ancestor_path in playbook_path.parts(reverse=True)[1:]:
        goodplay_yml_path = ancestor_path.join('.goodplay.yml')

        if goodplay_yml_path.check(file=True):
            return goodplay_yml_path


def get_default_goodplay_config():
    return dict(platforms=[])



# -*- coding: utf-8 -*-

import pytest

from goodplay.docker_support import DockerRunner


@pytest.fixture
def without_docker_env_vars(monkeypatch):
    monkeypatch.delenv('DOCKER_TLS_VERIFY', raising=False)
    monkeypatch.delenv('DOCKER_HOST', raising=False)
    monkeypatch.delenv('DOCKER_CERT_PATH', raising=False)
    monkeypatch.delenv('DOCKER_MACHINE_NAME', raising=False)


def test_client_negotiates_version(without_docker_env_vars, mocker):
    client_mock = mocker.patch('docker.Client', autospec=True)

    docker_runner = DockerRunner(ctx=None)
    docker_runner.client

    client_mock.assert_called_once_with(version='auto')


def test_client_uses_docker_host_env_var(without_docker_env_vars, monkeypatch, mocker):
    monkeypatch.setenv('DOCKER_HOST', 'tcp://192.168.10.2:2376')

    client_mock = mocker.patch('docker.Client', autospec=True)

    docker_runner = DockerRunner(ctx=None)
    docker_runner.client

    _, _, kwargs = client_mock.mock_calls[0]
    assert kwargs['base_url'] == 'tcp://192.168.10.2:2376'


def test_client_does_not_validate_hostname(without_docker_env_vars, mocker):
    mocker.patch('docker.Client', autospec=True)
    kwargs_from_env_mock = mocker.patch('docker.utils.kwargs_from_env', autospec=True)

    docker_runner = DockerRunner(ctx=None)
    docker_runner.client

    kwargs_from_env_mock.assert_called_once_with(assert_hostname=False)


def test_client_is_cached(mocker):
    client_mock = mocker.patch('docker.Client', autospec=True)

    docker_runner = DockerRunner(ctx=None)
    first_result = docker_runner.client

    assert id(docker_runner.client) == id(first_result)
    assert client_mock.call_count == 1



# -*- coding: utf-8 -*-

import re

import _pytest.junitxml


def patch_pytest_to_strip_file_extensions():
    if hasattr(_pytest.junitxml, '_py_ext_re'):
        # pytest>=2.9.1
        _pytest.junitxml._py_ext_re = re.compile(r'\.(?:py|yml)$')
    else:  # pragma: no cover
        # pytest<=2.9.0
        # monkeypatch mangle_testnames to remove .yml extension from playbook "class"
        mangle_testnames_orig = _pytest.junitxml.mangle_testnames

        def mangle_testnames_patch(names):
            names = mangle_testnames_orig(names)
            return [x[:-4] if x.endswith('.yml') else x for x in names[:-1]] + [names[-1]]
        _pytest.junitxml.mangle_testnames = mangle_testnames_patch



# -*- coding: utf-8 -*-

from goodplay.context import Platform
from goodplay.plugin import GoodplayPlaybookFile, GoodplayPlatform, GoodplayPlaybook, GoodplayTest


class AttrDict(dict):
    def __getattr__(self, attr):
        return self[attr]

    def __setattr__(self, attr, value):
        self[attr] = value


def test_nodeid_goodplay_playbook_file(tmpdir):
    playbook_path = tmpdir.join('test_playbook.yml')

    config = AttrDict(rootdir=tmpdir)
    session = AttrDict(non='empty')

    node = GoodplayPlaybookFile(ctx=None, fspath=playbook_path, config=config, session=session)
    assert node.nodeid == 'test_playbook.yml'


def test_nodeid_goodplay_platform_with_no_platform_is_same_as_parent(tmpdir):
    parent = AttrDict(
        config=AttrDict(rootdir=tmpdir),
        session=AttrDict(non='empty'),
        fspath=tmpdir.join('test_playbook.yml'),
        nodeid='test_playbook.yml'
    )

    node = GoodplayPlatform(platform=None, parent=parent)
    assert node.nodeid == 'test_playbook.yml'


def test_nodeid_goodplay_platform_with_platform_adds_platform(tmpdir):
    parent = AttrDict(
        config=AttrDict(rootdir=tmpdir),
        session=AttrDict(non='empty'),
        fspath=tmpdir.join('test_playbook.yml'),
        nodeid='test_playbook.yml'
    )

    platform = Platform(name='ubuntu', version='trusty')
    node = GoodplayPlatform(platform=platform, parent=parent)
    assert node.nodeid == 'test_playbook.yml::ubuntu:trusty'


def test_nodeid_goodplay_playbook_is_same_as_parent(tmpdir):
    parent = AttrDict(
        config=AttrDict(rootdir=tmpdir),
        session=AttrDict(non='empty'),
        fspath=tmpdir.join('test_playbook.yml'),
        nodeid='test_playbook.yml::ubuntu:trusty'
    )

    node = GoodplayPlaybook(name='test_playbook.yml', parent=parent)
    assert node.nodeid == 'test_playbook.yml::ubuntu:trusty'


def test_nodeid_goodplay_test_add_test_name(tmpdir):
    parent = AttrDict(
        config=AttrDict(rootdir=tmpdir),
        session=AttrDict(non='empty'),
        fspath=tmpdir.join('test_playbook.yml'),
        nodeid='test_playbook.yml::ubuntu:trusty'
    )

    task = AttrDict(name='host is reachable')

    node = GoodplayTest(task=task, parent=parent)
    assert node.nodeid == 'test_playbook.yml::ubuntu:trusty::host is reachable'



# -*- coding: utf-8 -*-

from cached_property import cached_property
import py.path
import yaml

from goodplay import ansible_support
from goodplay.config import get_goodplay_config


class GoodplayContext(object):
    def __init__(self, playbook_path, pytestconfig=None):
        self.playbook_path = playbook_path
        self.pytestconfig = pytestconfig

        self._temp_paths = []

    def _create_temporary_dir(self):
        temp_path = py.path.local.mkdtemp()
        self._temp_paths.append(temp_path)
        return temp_path

    @cached_property
    def config(self):
        return get_goodplay_config(self.playbook_path)

    @cached_property
    def platform_manager(self):
        platform_manager = PlatformManager.from_dicts(self.config['platforms'])

        if self.is_platform_matrix_requested:
            platforms = self.role_meta_info['galaxy_info'].get('platforms', [])
            for platform in platforms:
                name = platform['name']

                for version in platform['versions']:
                    platform_manager.select_platform_by_name_and_version(name, version)

        return platform_manager

    @cached_property
    def is_platform_matrix_requested(self):
        is_goodplay_platform_wildcard_used = \
            any(host.vars().get('goodplay_platform', None) == '*'
                for host in self.inventory.hosts())

        return self.is_role_playbook and is_goodplay_platform_wildcard_used

    @cached_property
    def platforms(self):
        return self.platform_manager.selected_platforms

    @cached_property
    def inventory_path(self):
        inventory_path = self.playbook_path.dirpath('inventory')

        if inventory_path.check():
            return inventory_path

    @cached_property
    def inventory(self):
        if self.inventory_path:
            return ansible_support.Inventory(self.inventory_path)

    @cached_property
    def extended_inventory_path(self):
        extended_inventory_path = self._create_temporary_dir()

        extended_inventory_path.join('inventory').mksymlinkto(self.inventory_path)

        if self.inventory_path.check(dir=True):
            group_vars_path = self.inventory_path.join('group_vars')

            if group_vars_path.check(dir=True):
                extended_inventory_path.join('group_vars').mksymlinkto(group_vars_path)

            host_vars_path = self.inventory_path.join('host_vars')

            if host_vars_path.check(dir=True):
                extended_inventory_path.join('host_vars').mksymlinkto(host_vars_path)

        # when inventory_path is a file, group_vars and host_vars directories
        # are handled implicitly as they are beside the executed playbook

        return extended_inventory_path

    @cached_property
    def playbook(self):
        if self.inventory_path:
            return ansible_support.Playbook(self)

    @cached_property
    def is_role_playbook(self):
        return bool(self.role_path)

    @cached_property
    def role_path(self):
        for ancestor_path in self.playbook_path.parts(reverse=True)[1:]:
            if ancestor_path.basename == 'tests':
                role_path = ancestor_path.dirpath()
                is_role_path = role_path.join('meta', 'main.yml').check(file=True)

                if is_role_path:
                    return role_path
                break

    @cached_property
    def role_meta_info(self):
        return yaml.safe_load(self.role_path.join('meta', 'main.yml').read())

    @cached_property
    def role_under_test_roles_path(self):
        role_under_test_roles_path = self._create_temporary_dir()

        role_under_test_roles_path.join(self.role_path.basename).mksymlinkto(self.role_path)

        return role_under_test_roles_path

    @cached_property
    def installed_roles_path(self):
        return self._create_temporary_dir()

    @cached_property
    def use_local_roles(self):
        return self.pytestconfig.getoption('use_local_roles')

    def release(self):
        for temp_path in reversed(self._temp_paths):
            temp_path.remove(ignore_errors=True)


class PlatformManager(object):
    def __init__(self, available_platforms):
        self.available_platforms = available_platforms

        self.selected_platforms = []

    @classmethod
    def from_dicts(cls, platform_dicts):
        available_platforms = [Platform(**platform_dict) for platform_dict in platform_dicts]
        return PlatformManager(available_platforms)

    def find_by_id(self, id):
        if ':' not in id:
            return None

        name, version = id.split(':', 1)

        return self.find_by_name_and_version(name, version)

    def find_by_name_and_version(self, name, version):
        platform_template = Platform(name, version)

        try:
            index = self.available_platforms.index(platform_template)
            return self.available_platforms[index]
        except ValueError:
            pass

    def select_platform_by_name_and_version(self, name, version):
        platform = self.find_by_name_and_version(name, version)

        if platform and platform not in self.selected_platforms:
            self.selected_platforms.append(platform)


class Platform(object):
    def __init__(self, name, version, image=None, **kwargs):
        self.name = name
        self.version = version
        self.image = image

    def __str__(self):
        return '{0!s}:{1!s}'.format(self.name, self.version)

    def __eq__(self, other):
        return (isinstance(other, self.__class__) and str(self) == str(other))

    def __ne__(self, other):
        return not self.__eq__(other)



from goodplay_helpers import smart_create


def test_something(testdir):
    smart_create(testdir.tmpdir, '''
    ## .goodplay.yml
    platforms:
      - name: EL
        version: 6
        image: busybox:latest

    ## meta/main.yml
    galaxy_info:
      author: Benjamin Schwarze
      platforms:
        - name: EL
          versions:
            - 6
    dependencies: []

    ## tasks/main.yml
    - ping:

    ## tests/inventory
    host1 goodplay_platform=*
    #host1 goodplay_image=*

    ## tests/test_playbook.yml
    - hosts: host1
      gather_facts: no
      tasks:
        - name: host is reachable
          ping:
          tags: test
    ''')

    result = testdir.inline_run('-s')
    result.assertoutcome(passed=1)
